from pathlib import Path

from setuptools import setup, find_packages

BASE_DIR = Path(__file__).parent.resolve(strict=True)
VERSION = '0.0.1'
PACKAGE_NAME = 'aiovty'
PACKAGES = [p for p in find_packages() if not p.startswith('tests')]


def get_description():
    return (BASE_DIR / 'README.md').read_text()


if __name__ == '__main__':
    setup(
        version=VERSION,
        name=PACKAGE_NAME,
        description='A python asyncio implementation for VTY protocol',
        long_description=get_description(),
        long_description_content_type='text/markdown',
        packages=PACKAGES,
        author='Matan Perelman',
        classifiers=[
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
        ],
        url='https://github.com/matan1008/aiovty',
        project_urls={
            'aiovty': 'https://github.com/matan1008/aiovty'
        },
        tests_require=['pytest', 'pytest_asyncio'],
    )



import asyncio


class AioVtyClient:
    def __init__(self, name: str):
        self.name = name
        self.prompt_char = '>'
        self.node = ''
        self.reader = None
        self.writer = None

    async def connect(self, host, port):
        self.reader, self.writer = await asyncio.open_connection(host, port)
        return await self.read()

    async def close(self):
        self.writer.close()
        await self.writer.wait_closed()

    async def enable(self):
        await self.write(b'enable')
        self.prompt_char = '#'
        await self.read()

    async def disable(self):
        await self.write(b'disable')
        self.prompt_char = '>'
        await self.read()

    async def command(self, command):
        await self.write(command)
        data = (await self.read())[len(command):].strip(b'\r\n')
        return data

    async def configure_terminal(self):
        await self.command(b'configure terminal')

    async def end(self):
        await self.command(b'end')

    async def read(self):
        # Read until a new line feed.
        node_indicator = b''
        data = b''
        while node_indicator not in (b'(', self.prompt_char.encode()):
            data += node_indicator
            data += (await self.reader.readuntil(self.name.encode()))[:-len(self.name.encode())]
            node_indicator = await self.reader.read(1)
        if node_indicator == self.prompt_char.encode():
            # Node mode exited.
            self.node = ''
            await self.reader.read(len(' '))
            return data
        # Update node.
        self.node = (await self.reader.readuntil(b')'))[:-len(b')')].decode()
        await self.reader.readuntil(f'{self.prompt_char} '.encode())
        return data

    async def write(self, line):
        self.writer.write(line + b'\n')
        await self.writer.drain()
